/*     crystal                                                           */
/*                                                                       */
/* Routine to read crystallographic information such as unit cell, space */
/* group, and atom positions to generate POLYHEDRON, SPHERE and BOND     */
/* geometry for AVS, or to create POV and VRML scenes.                   */
/*                                                                       */
/* Coded by:   L.W. Finger                                               */
/*            Geophysical Laboratory                                     */
/*            Carnegie Institution of Washington                         */
/*            Washington DC 20015                                        */
/*            finger@gl.ciw.edu                                          */
/*                                                                       */
/* Original Version: April 92                                            */
/*                   Revised January 1993 to add bonds.                  */
/*                   Converted to AVS Module February 1993.              */
/*                                                                       */
/*                                                                       */
/* Converted to standalone POV scene file generator October 1994/March 1995 */
/* by M. Kroeker, Organic Chemistry, Technische Hochschule Darmstadt,    */
/* Petersenstr. 22, D-64287 Darmstadt, Germany                           */
/* martin@oc2.oc.chemie.th-darmstadt.de                                  */
/*                                                                       */
/* Two versions merged  July 1995                                        */
/* VRML code added September 1995 and improved March 1996 M.Kroeker      */
/* ===================================================================== */
/*  AVS parameters                                                       */
/* param 0 "X_Boxlim" slider 4.00000  1.00000 30.0000                    */
/* param 1 "Y_Boxlim" slider 4.00000  1.00000 30.0000                    */
/* param 2 "Z_Boxlim" slider 4.00000  1.00000 30.0000                    */
/* param 3 "X_Origin" slider 0.500000 0.00000 1.00000                    */
/* param 4 "Y_Origin" slider 0.500000 0.00000 1.00000                    */
/* param 5 "Z_Origin" slider 0.500000 0.00000 1.00000                    */
/* param 6 "Unit_Cell" radio_buttons "Yes" "Yes No" ":"                  */
/* param 7 "Sphere_Mult" slider 1.00 0.25 4.0                            */
/* param 8 "Bond_Mult" slider 1.00 0.25 4.0                              */
/* param 9 "Str_File" browser "" "" ":"                                  */
/* End of Module Description Comments                                    */

#define MEM_DEFS_ENABLE 1

#include <stdio.h>
#include <string.h>
#include <math.h>
#if defined _POV_ || defined VRML
#include <stdlib.h>

void usage();
#else
#include <avs/geom.h>
#include <avs/port.h>
#include <avs/avs.h>
#endif
int crystal_compute();
void read_inp();
void print_sym();
void make_bmat();
void symop();
void draw_cell();
void get_atom_id();
void find_all_in_box();
float dist();
void expand_atom();
void add_to_list();
void build_poly_list();
void generate_poly();
void generate_bonds();
void generate_spheres();
void generate_planes();
#ifdef VRML
void plot_vrml_poly();
#endif
 
int ss[24][3][3];               /* rotation operators for symmetry */
int sys;                        /* crystal system indicator */
int nbr;                        /* Bravais lattice indicator */
int ns;                         /* center of symmetry indicator */
int ng;                         /* number of rotation operations to specify group */
float lat_con[6];               /* lattice constants */
int nlat;                       /* number of lattice positions */
float lat_pos[4][3];            /* coordinates of lattice points */
float ts[24][3];                /* translation operators */
char spg[3][4];                 /* space group code */
char savedpath[100];            /* path at start of program */
int npoly;                      /* number of different types of polyhedra */
int natom;                      /* number of different types of atoms */
int nsphere;                    /* number of different types of spheres */
int nbond;                      /* number of different types of bonds */
int nvert;                      /* number of vertices in list */
int nplane;
float s_vert[20000*3];          /* saved coordinates of vertices */
float atom_xyz[3000][3];        /* atomic coordinates */
char atom_l[3000][2];           /* atom labels */
char File_Name[100];            /* Modified Input File Name */
int atom_n[3000];               /* atom number */
float boxlim[3];                /* half limits of plotting box */
float Sphere_Mult;              /* Multiplier for Sphere Size */
float Bond_Mult;		/* Multiplier for Bond Size */
char sphere_l[20][2];           /* sphere label */
float sphere_size[20];          /* sphere size */
char poly_l[3000][2];           /* polyhedral center label */
char plane_l[3000][2];
float poly_size[3000];          /* size of polyhedron */
float plane_size[3000];
float origin[3];                /* position of plotting origin */
char bond_l1[20][2];            /* label for one end of bond */
char bond_l2[20][2];            /* label for other end */
char *Str_File_Inp;             /* Input File Name */
float bond_size[20];            /* bond diameter */
float bond_min[20];             /* minimum bond distance */
float bond_max[20];             /* maximum bond distance */
float b_mat[3][3];              /* metric tensor */
float cell_xyz[192][3];         /* storage for generated equivalent positions */
int ncell;                      /* number of atoms in equipoint */
int draw_list;                  /* number of items in polygon draw list */
int poly_list[20000];           /* storage for polygon corner pointers */
int vertex_list[30];            /* storage for list of vertices about a cation */
int numb_list;                  /* number in polygon list */
int addflag;			/* forces addition of objects outside box */

char sphere_col[20][30];        /* sphere colors */
char poly_col[3000][30];        /*polyhedron colors */
char plane_col[3000][30];        /*polyhedron colors */
float rad_cell;			/* radius of unit cell framebars */
char col_cell[30];		/* color and properties of unit cell framebars */
int edges;			/* draw thin black lines around the edges of polyhedra*/
float rad_edge;			/* radius of cylinders on polyhedron edges */
char col_edge[30];		/* color and properties of polyhedron edges */
char col_bond[20][30];		/* color and properties of bonds */
float xrot,yrot,zrot;           /* view rotation angles */

#if defined _POV_ || defined VRML

float cryst_lim[6];             /* limits of view box in cell coords */
float o_vert[20000*3];           /* crystal coordinates of saved vertices */
float X_Boxlim;
float Y_Boxlim;
float Z_Boxlim;
float X_Origin;
float Y_Origin;
float Z_Origin;
int Unit_Cell;
char *Str_File;
float SpMult;
float BndMult;

#endif

FILE *fpin,*fpout,*flout;

#if defined _POV_ || defined VRML
int main(int argc,char *argv[])

{
    int i,boxflag;
    int doit;
    
    X_Origin = 0.5;
    Y_Origin = 0.5;
    Z_Origin = 0.5;
    X_Boxlim = 10.;
    Y_Boxlim = 10.;
    Z_Boxlim = 10.;
    cryst_lim[0]=-100.;
    cryst_lim[1]=-100.;
    cryst_lim[2]=-100.;
    cryst_lim[3]=100.;
    cryst_lim[4]=100.;
    cryst_lim[5]=100.;
    xrot=0.;
    yrot=0.;
    zrot=0.;
    boxflag=0.;
    Unit_Cell = 1;
    SpMult = 1.;
    BndMult = 1.;
    edges =0;    
    strcpy(col_cell,"Black");    
    /* Parse arguments */
    if (argc <2) usage();

for (i = 1; i < argc; i++) {
	if (!strcmp(argv[i], "-n"))
	    Unit_Cell = 0;
	else if (!strcmp(argv[i], "-h")) usage();
	else if (!strcmp(argv[i], "-?")) usage();
	else if (!strcmp(argv[i], "-o")){
            i++	;
	    X_Origin = atof(argv[i]);
	    i++;
	    Y_Origin = atof(argv[i]);
	    i++;
	    Z_Origin = atof(argv[i]);
	    }
	else if (!strcmp(argv[i], "-b")){
            i++	;
	    X_Boxlim = atof(argv[i]);
	    i++;
	    Y_Boxlim = atof(argv[i]);
	    i++;
	    Z_Boxlim = atof(argv[i]);
	    boxflag=1;
	    }
	else if (!strcmp(argv[i], "-p")){
            i++	;
	    cryst_lim[0] = atof(argv[i]);
	    i++;
	    cryst_lim[3] = atof(argv[i]);
	    i++;
	    cryst_lim[1] = atof(argv[i]);
            i++	;
	    cryst_lim[4] = atof(argv[i]);
	    i++;
	    cryst_lim[2] = atof(argv[i]);
	    i++;
	    cryst_lim[5] = atof(argv[i]);
	    if (boxflag ==0) X_Boxlim=Y_Boxlim=Z_Boxlim=100.;
	    }
	else if (!strcmp(argv[i], "-v")){
            i++	;
	    xrot = atof(argv[i]);
	    i++;
	    yrot = atof(argv[i]);
	    i++;
	    zrot = atof(argv[i]);
	    }
	else 
	Str_File=argv[i];
}
        doit=crystal_compute();      /* convert to crystal.pov */
        exit(0);
} /* end main */


void usage()
{
  fprintf(stderr,"usage: crystal [-n] [-o Xo Yo Zo] [-b Xb Yb Zb] [-v Xr Yr Zr] <file>\n");
  fprintf(stderr,"-n no display of unit cell\n");
  fprintf(stderr,"-o x-, y-, z-coordinates of origin (0.5,0.5,0.5)\n");
  fprintf(stderr,"-b x-, y-, z-limits of view box in angstroms(10.,10.,10.)\n");
  fprintf(stderr,"-p a-, b-, c-ranges of packing box (-100., 100., -100., 100., -100., 100.)\n");
  fprintf(stderr,"-v xrot yrot zrot : View rotation angles (0. 0. 0.)\n");
  exit(1);
}

int crystal_compute()

{
    int docell,i;
    char outfil[100];

    if (Str_File == NULL) return(0);  /* Fail immediately if filename NULL */
    origin[0] = X_Origin;
    origin[1] = Y_Origin;
    origin[2] = Z_Origin;
    boxlim[0] = X_Boxlim;
    boxlim[1] = Y_Boxlim;
    boxlim[2] = Z_Boxlim;
    docell = Unit_Cell;
    rad_cell=0.02;  
    Sphere_Mult = SpMult;
    Bond_Mult = BndMult;
    Str_File_Inp = Str_File;
    flout = fopen("crystal.out","w");

#else
 
/* *****************************************/
/*  Module Description                     */
/* *****************************************/
int crystal_desc()
{

	int out_port, param, iresult, i, inch;
	char filestring[100];
	extern int crystal_compute();

	AVSset_module_name("Crystal", MODULE_DATA);

	out_port = AVScreate_output_port("Geometry" , "geom");
	AVSautofree_output(out_port);

	/* Parameter Specifications                */
	param = AVSadd_float_parameter("X_Boxlim", 4.0, 1.0, 30.0);
	AVSconnect_widget(param, "slider");
	param = AVSadd_float_parameter("Y_Boxlim", 4.0, 1.0, 30.0);
	AVSconnect_widget(param, "slider");
	param = AVSadd_float_parameter("Z_Boxlim", 4.0, 1.0, 30.0);
	AVSconnect_widget(param, "slider");
	param = AVSadd_float_parameter("X_Origin", 0.5, -1.0, 2.0);
	AVSconnect_widget(param, "slider");
	param = AVSadd_float_parameter("Y_Origin", 0.5, -1.0, 2.0);
	AVSconnect_widget(param, "slider");
	param = AVSadd_float_parameter("Z_Origin", 0.5, -1.0, 2.0);
	AVSconnect_widget(param, "slider");
	param = AVSadd_parameter("Unit_Cell", "boolean", 0 , 0 , 0 );
	AVSconnect_widget(param, "toggle");
	param = AVSadd_float_parameter("Sphere_Mult", 1.0, 0.25, 4.0);
	AVSconnect_widget(param, "slider");
	param = AVSadd_float_parameter("Bond_Mult", 1.0, 0.25, 4.0);
/* get current path */
	system("pwd > temp");
	fpin = fopen("temp","r");
	for (i=0;inch != '\n'; ) {
            inch = fgetc(fpin);
            filestring[i++] = inch;
        }
        filestring[--i] = '/';
	filestring[i+1] = '\0';
        fclose(fpin);
	remove("temp");
        strcpy(savedpath,filestring);
	AVSconnect_widget(param, "slider");
	param = AVSadd_parameter("Str_File", "string", filestring, "", ":");
	AVSconnect_widget(param, "browser");
        AVSadd_parameter_prop(param,"height","integer",20);

	AVSset_compute_proc(crystal_compute);
	return(1);
}
 
/* *****************************************/
/* Module Compute Routine                  */
/* *****************************************/
int crystal_compute( output, X_Boxlim, Y_Boxlim, Z_Boxlim, X_Origin, Y_Origin, 
		     Z_Origin, Unit_Cell, SpMult, BndMult, Str_File)
        GEOMedit_list *output;
	float *X_Boxlim;
	float *Y_Boxlim;
	float *Z_Boxlim;
	float *X_Origin;
	float *Y_Origin;
	float *Z_Origin;
	int *Unit_Cell;
	char *Str_File;
	float *SpMult;
        float *BndMult;

{
    int *docell;
    GEOMobj *obj1;            /* Dummy TOP Object */

    if (strcmp(savedpath,Str_File) == 0 ) return(0);  /* Fail immediately if filename only path */

/* Copy Parameters from widgets into local storage - Match up with former
             stand-alone code */

    origin[0] = *X_Origin;
    origin[1] = *Y_Origin;
    origin[2] = *Z_Origin;
    boxlim[0] = *X_Boxlim;
    boxlim[1] = *Y_Boxlim;
    boxlim[2] = *Z_Boxlim;
    docell = Unit_Cell;
    Sphere_Mult = *SpMult;
    Bond_Mult = *BndMult;
    Str_File_Inp = Str_File;
   
    if (AVSparameter_changed("Str_File") != 0) 
    {
	flout = fopen("crystal.out","w");

#endif   /* def _POV_  */

	natom = 0;
	nsphere = npoly = nbond = nplane = 1;
	poly_size[0] = 2.5;
	poly_l[0][0] = poly_l[0][1] = ' ';
	plane_size[0] = 2.5;
	plane_l[0][0] = plane_l[0][1] = ' ';
	sphere_size[0] = 0.0;
	sphere_l[0][0] = sphere_l[0][1] = ' ';
	if ((fpin=fopen(Str_File,"r")) == NULL) {
	    fprintf(flout,"Cannot Open Input File");
	    return(0);
	}

#if defined _POV_ || defined VRML 

/*	outfil=Str_File;*/
        strcpy(outfil,Str_File);
	for (i=1;i<strlen(outfil);i++){
	if (outfil[i] == '.' || outfil[i] ==' ')
	break;
	}
	outfil[i]='\0';
#ifdef VRML
	strcat(outfil,".wrl");
#else
	strcat(outfil,".pov");
#endif
	if ((fpout=fopen(outfil,"w")) == NULL) {
	    fprintf(flout,"Cannot open output file\n");
	    return(0);
	}
	fprintf(flout,"\n\nCrystal Structure Drawing Program V1.2\n\n");
	fprintf(flout,"\n\nModified POV Version V1.0\n\n");
	fprintf(flout,"\n********************* Input: ************************\n");

#else

	fprintf(flout,"\n\nCrystal Structure Drawing Program V1.2\n\n");
	fprintf(flout,"\n********************* Input: ************************\n");

#endif

	read_inp();
	fprintf(flout,"*****************************************************\n\n");
	print_sym();    /* print symmetry information */
	fclose(fpin);         /* close file */
#if defined _POV_ || defined VRML

#ifdef VRML
fprintf(fpout,"#VRML V1.0 ascii\n");
    fprintf(fpout,"# Scene file created by POVcrystal from %s */\n",Str_File_Inp);
    fprintf(fpout,"# with command line options: -b %6.2f %6.2f %6.2f -o %5.2f %5.2f %5.2f */\n",
    boxlim[0],boxlim[1],boxlim[2],origin[0],origin[1],origin[2]); 
    fprintf(fpout,"#  -p %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f -v %5.2f %5.2f %5.2f */\n",
    cryst_lim[0],cryst_lim[3],cryst_lim[1],cryst_lim[4],cryst_lim[2],cryst_lim[5],xrot,yrot,zrot); 


fprintf(fpout,"Separator {\n   Material {\n  ambientColor    0.1 0.1 0.1\n");
fprintf(fpout,"        diffuseColor    1 1 1\n   specularColor   0.9 0.9 0.9\n");
fprintf(fpout,"        shininess       0.8\n    }\n\n");
fprintf(fpout,"    PerspectiveCamera {\n        position        2.646 -2.747 108.446\n");
fprintf(fpout,"        orientation     0 0 1  0\n        focalDistance   103.662\n");
fprintf(fpout,"        heightAngle     0.285796\n    }\n\n");

#else
    fprintf(fpout,"/* Scene file created by POVcrystal from %s */\n",Str_File_Inp);
    fprintf(fpout,"/* with command line options: -b %6.2f %6.2f %6.2f -o %5.2f %5.2f %5.2f */\n",
    boxlim[0],boxlim[1],boxlim[2],origin[0],origin[1],origin[2]); 
    fprintf(fpout,"/*  -p %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f -v %5.2f %5.2f %5.2f */\n",
    cryst_lim[0],cryst_lim[3],cryst_lim[1],cryst_lim[4],cryst_lim[2],cryst_lim[5],xrot,yrot,zrot); 

    fprintf(fpout,"#include \"colors.inc\"\n");
    fprintf(fpout,"camera{\n");
    fprintf(fpout,"location< 0 ,0, %f>\n",-30.*lat_con[2]);
    fprintf(fpout,"up <0, 0, 0.1>\n");
    fprintf(fpout,"right <0.1, 0, 0>\n");
    fprintf(fpout,"look_at <0,0,0>\n");
    fprintf(fpout,"}\n");
    fprintf(fpout,"background{color White}\n");
    fprintf(fpout,"light_source {< 0, 0, %f> color red 4.0 green 4.0 blue 4.0 }\n",-40.*boxlim[2]);
    fprintf(fpout,"object{ union{ \n");
#endif

    make_bmat(lat_con);
    draw_cell(docell);      /* generate unit-cell corners and draw it */

#else

    }

    make_bmat(lat_con);
    *output = GEOMinit_edit_list(*output);  /* initialize list of changes */
/* create empty TOP object */
    obj1 = GEOMcreate_obj(GEOM_POLYTRI,GEOM_NULL);
    strcpy(File_Name,Str_File_Inp);
    GEOMedit_geometry(*output,File_Name,obj1);
    GEOMdestroy_obj(obj1);      /* destroy object */
    draw_cell(output,docell);      /* generate unit-cell corners and draw it */

#endif

    get_atom_id();        /* rework atom id numbers to pointers */
    nvert = 0;            /* empty vertex list */

#if defined _POV_ || defined VRML

    generate_spheres();   /* generate all spheres to be drawn within box */
    generate_bonds();     /* generate all bonds to be drawn within box */
    generate_poly();      /* generate all polyhedra to be drawn */
    generate_planes();
#ifdef _POV_
    fprintf(fpout,"} rotate < %f, %f, %f>\n",xrot,yrot,zrot);
#else
    fprintf(fpout,"   }\n }\n");
#endif
    fprintf(fpout,"}\n");
    fclose(fpout);
    fclose(flout);

#else

    generate_spheres(output); /* generate all spheres to be drawn within box */
    generate_bonds(output);     /* generate all bonds to be drawn within box */
    generate_poly(output);      /* generate all polyhedra to be drawn */
    if (AVSparameter_changed("Str_File") != 0)
	fclose(flout);

#endif

    return(1);
}

/* ***********************************************************************/
/* Initialization for modules contained in this file.                    */
/* ***********************************************************************/

#if !defined _POV_ && !defined VRML

static int ((*mod_list[])()) = {
	crystal_desc
};
#define NMODS (sizeof(mod_list) / sizeof(char *))

void AVSinit_modules()
{
	AVSinit_from_module_list(mod_list, NMODS);
}

#endif
 

/* ************************************************************** */
/* ************************************************************** */

void get_atom_id()

/* routine to rework atom numbers to point at polyhedron or sphere
   characteristics list */

{
    int i,j,ind;                /* local variables */

    for (i=0;i<natom;++i) {
	ind = 0;
	for (j=1;j<nsphere;++j) {
	    if((atom_l[i][0] == sphere_l[j][0]) &&
	    (atom_l[i][1] == sphere_l[j][1])) {
		atom_n[i] = -j; /* atom is in sphere lst - no. is negative */
		ind = 1;        /* set indicator */
	    }
	}
	if (ind == 0) {
	    for (j=1;j<npoly;++j) {
		if((atom_l[i][0] == poly_l[j][0]) &&
		(atom_l[i][1] == poly_l[j][1])) {
		    atom_n[i] = j;      /* atom is in poly. list */
		    ind = 1;            /* set indicator */
		}
	    }
	}
	if (ind == 0) {
	    for (j=1;j<nplane;++j) {
		if((atom_l[i][0] == plane_l[j][0]) &&
		(atom_l[i][1] == plane_l[j][1])) {
		    atom_n[i] = j+10000;      /* atom is in plane list */
		    ind = 1;            /* set indicator */
		}
	    }
	}
	if(ind == 0) atom_n[i] = 0;  /* if not sphere or polyhedron - use
					default polyhedron (#0) */
    }
}

/* ************************************************************** */
/* ************************************************************** */
 
void read_inp()

/* routine to read and process input file */

{
    char input[100];               /* input buffer for data read from file */
    char string[10];               /* temporary string */
    int intype;                    /* type of datum read */
    int i;                         /* temporary */

    while (!feof(fpin)) {
	if(fgets(input,100,fpin)) {             /* read a line */
	    fprintf(flout,"*         %s",input);       /* echo to stdout */
	} else {
	    fprintf(flout,"Error reading file.");
	    exit(1);
	}
	intype = 0;
	if(strncmp(input,"title",5) == 0)intype = 1;
	if(strncmp(input,"cell",4) == 0)intype = 2;
	if(strncmp(input,"sgrp",4) == 0)intype = 3;
	if(strncmp(input,"spgp",4) == 0)intype = 3;
	if(strncmp(input,"spgr",4) == 0)intype = 3;
	if(strncmp(input,"atom",4) == 0)intype = 6;
	if(strncmp(input,"polysz",6) == 0)intype = 7;
	if(strncmp(input,"sphere",6) == 0)intype = 8;
	if(strncmp(input,"end",3) == 0)intype = 9;
	if(strncmp(input,"bond",4) == 0)intype = 11;
	if(strncmp(input,"box",3) == 0)intype = 12;
	if(strncmp(input,"edges",5) == 0)intype = 13;
        if(strncmp(input,"view",4) == 0) intype =14;
        if(strncmp(input,"plane",5)==0) intype=15;
	switch (intype) {
	    case 1: break;      /* ignore title except for listing */
	    case 2: for (i=3;i<=5;++i) lat_con[i] = 0.;
		    sscanf(input,"%s %f %f %f %f %f %f",string,&lat_con[0]
		    ,&lat_con[1],&lat_con[2],&lat_con[3],&lat_con[4]
		    ,&lat_con[5]);
		    break;
	    case 3: symop(input);   /* process space group information */
		    break;
	    case 6: sscanf(input,"%5s %c%c %d %f %f %f",string,
			 &atom_l[natom][0],&atom_l[natom][1],
			 &atom_n[natom],&atom_xyz[natom][0],
			 &atom_xyz[natom][1],&atom_xyz[natom][2]);
		    natom++;
		    break;
	    case 7: sscanf(input,"%7s %c%c %f %30c",string,
			 &poly_l[npoly][0],&poly_l[npoly][1],
			 &poly_size[npoly],poly_col[npoly]);
		    npoly++;
		    break;
	    case 8: sscanf(input,"%7s %c%c %f %30c",string,
			 &sphere_l[nsphere][0],&sphere_l[nsphere][1],
			 &sphere_size[nsphere],sphere_col[nsphere]);
		    nsphere++;
		    break;
	    case 9: return;
	    case 11: sscanf(input,"%5s %c%c %c%c %f %f %f %30c",string,
			 &bond_l1[nbond][0],&bond_l1[nbond][1],
			 &bond_l2[nbond][0],&bond_l2[nbond][1],
			 &bond_size[nbond],&bond_min[nbond],&bond_max[nbond],
			 col_bond[nbond]);
	    	    if (strlen(col_bond[nbond]) == 0) strcpy(col_bond[nbond],"Grey");
		    nbond++;
		    break;
	    case 12: sscanf(input,"%3s %f %30c",string,&rad_cell,col_cell);
	    	    if (strlen(col_cell) == 0) strcpy(col_cell,"Black");
	    	    break;	    
	    case 13: sscanf(input,"%5s %f %30c",string,&rad_edge,col_edge);
                    edges=1;
                    if (rad_edge == 0) rad_edge=0.01;
	    	    if (strlen(col_edge) == 0) strcpy(col_edge,"Black");
	    	    break;	    
	    case 14: sscanf(input,"%4s %f %f %f",string,&xrot,&yrot,&zrot);
	            break;	
	    case 15: sscanf(input,"%7s %c%c %f %30c",string,
			 &plane_l[nplane][0],&plane_l[nplane][1],
			 &plane_size[nplane],plane_col[nplane]);
		    nplane++;
		    break;
	    default: fprintf(flout,"Previous line did not start with a recognizable keyword - ignored\n"); 
	    break;
	}
    }
}

/* ************************************************************** */
/* ************************************************************** */

void make_bmat(lat_con)

float lat_con[6];

{
    float rad = 57.29578;
    float snga,csal,csbe,csga;
    int i,j,k;

    for (i=3;i<=5;++i) if(lat_con[i] <= 0) lat_con[i] = 90.0;
    if (sys == 5) lat_con[5] = 120.0;
    for (i=0;i<=2;++i)
	for (j=0;j<=2;++j) b_mat[j][i] = 0.0; /* initialize matrix */
    b_mat[0][0] = 1.0;
    csal = cos(lat_con[3]/rad);
    csbe = cos(lat_con[4]/rad);
    csga = cos(lat_con[5]/rad);
    snga = sin(lat_con[5]/rad);
    b_mat[0][1] = csga;
    b_mat[1][1] = snga;
    b_mat[0][2] = csbe;
    b_mat[1][2] = (csal-csbe*csga)/snga;
    b_mat[2][2] = sqrt(1.0 - b_mat[0][2] * b_mat[0][2] - b_mat[1][2] * b_mat[1][2]);
    for (i=0;i<=2;++i)
	for (j=0;j<=2;++j) b_mat[j][i] = b_mat[j][i] * lat_con[i];
}
    
/* ************************************************************** */
/* ************************************************************** */
void print_sym()

/* print symmetry information */

{
    int i,j,k;

    switch (sys)  {
	case 1: fprintf(flout,"Triclinic - ");break;
	case 2: fprintf(flout,"Monoclinic - ");break;
	case 3: fprintf(flout,"Orthorhombic - ");break;
	case 4: fprintf(flout,"Tetragonal - ");break;
	case 5: fprintf(flout,"Hexagonal - ");break;
	case 6: fprintf(flout,"Cubic - ");
    }
    if (ns == 0) {
	fprintf(flout,"Centric - ");
    } else {
	fprintf(flout,"Acentric - ");
    }
    switch (nbr) {
	case 1: fprintf(flout,"P lattice"); break;
	case 2: fprintf(flout,"A-centered");break;
	case 3: fprintf(flout,"B-centered");break;
	case 4: fprintf(flout,"C-centered");break;
	case 5: fprintf(flout,"F-centered");break;
	case 6: fprintf(flout,"I-centered");break;
	case 7: fprintf(flout,"R-centered");break;
	default: fprintf(flout,"**** Unknown Lattice ***");
    }
    fprintf(flout," with %d Symmetry Operators\n\n",ng);
    for (i=1;i<=ng;++i) {
	for (j=1;j<=3;++j) {
	    for (k=1;k<=3;++k) {
		if (ss[i-1][j-1][k-1] != 0) {
		    if(ss[i-1][j-1][k-1] > 0) { fprintf(flout,"+");
			} else {fprintf(flout,"-");}
		    switch (k) {
			case 1: fprintf(flout,"x");break;
			case 2: fprintf(flout,"y");break;
			case 3: fprintf(flout,"z");
		    }
		}
	    }
	    if(ts[i-1][j-1] != 0.0) {
		if(ts[i-1][j-1] == 0.5) fprintf(flout,"+1/2");
		if(ts[i-1][j-1] == 0.25) fprintf(flout,"+1/4");
		if(ts[i-1][j-1] == 0.75) fprintf(flout,"+3/4");
		if(fabs(ts[i-1][j-1] - 0.3333333) < 0.00001 ) fprintf(flout,"+1/3");
		if(fabs(ts[i-1][j-1] - 0.66666667) < 0.00001 ) fprintf(flout,"+2/3");
		if(fabs(ts[i-1][j-1] - 0.16666667) < 0.00001 ) fprintf(flout,"+1/6");
		if(fabs(ts[i-1][j-1] - 0.83333333) < 0.00001 ) fprintf(flout,"+5/6");
	    }
	    if (j != 3) fprintf(flout,",");
	}
	if (i != ng)
	    if (i % 4 != 0) {fprintf(flout," ; ");} else {fprintf(flout,"\n");}
    }
    fprintf(flout,"\n");
}

/* ************************************************************** */
/* ************************************************************** */
void add_vert(vert)

float vert[3];                  /* input coordinates of vertices */

/* procedure to add the vertices in 'vert' to the current list if the
   vertex is contained within the limits of the box */

{

    float c_vert[3];                    /* vertex in cartesian coords */
    int i,j;                            /* loop variables */

#if defined _POV_ || defined VRML
    
/* MK 3.1.95 check if within pack range */
  if (addflag != 1)
      for (i=0;i<=2;i++)
        if ((vert[i]  < cryst_lim[i]) || (vert[i] > cryst_lim[i+3] )) 
          return; 
#endif
    for (i=0;i<=2;++i) {
	c_vert[i] = 0.0;
	for (j=0;j<=2;++j)
	    c_vert[i] = c_vert[i] + b_mat[i][j] * (vert[j] - origin[j]);
    }
#if defined _POV_ || defined VRML

    if ((addflag == 1) ||((fabs(c_vert[0]) <= boxlim[0]) && 
      (fabs(c_vert[1]) <= boxlim[1]) && (fabs(c_vert[2]) <= boxlim[2]))) {
	for (i=0;i<=2;++i){
	    s_vert[3*nvert + i] = c_vert[i];
	    o_vert[3*nvert + i] = vert[i]; 
        }
	nvert++;
    }
#else
    if ((fabs(c_vert[0]) <= boxlim[0]) && (fabs(c_vert[1]) <= boxlim[1])
	&& (fabs(c_vert[2]) <= boxlim[2])) {
	for (i=0;i<=2;++i)
	    s_vert[3*nvert + i] = c_vert[i];
	nvert++;
    }
#endif
}

/* ************************************************************** */
/* ************************************************************** */
#if defined _POV_ || defined VRML
void draw_cell(docell)
int docell;        /* non-zero if unit-cell to be drawn */

{
#else
void draw_cell(output,docell)
GEOMedit_list *output;
int *docell;        /* true if unit-cell to be drawn */

/* procedure to add unit-cell outline to geometry file */

{
    GEOMobj *obj;             /* Geometry object */
#endif
    float vert[3];
    float white[3] = {1.0,1.0,1.0};
    float black[3] = {0.0,0.0,0.0};
    int i;

    nvert = 0;                  /* initialize vertex list */
    addflag = 1;
    for (i=0;i<=2;++i)
	vert[i] = 0.0;
    add_vert(vert);             /* place origin at 0,0,0 in list */
    vert[2] = 1.0;add_vert(vert);             /* place 0,0,1 in list */
    vert[1] = 1.0;add_vert(vert);             /* place 0,1,1 in list */
    vert[2] = 0.0;add_vert(vert);             /* place 0,1,0 in list */
    vert[1] = 0.0;add_vert(vert);             /* place 0,0,0 in list */
    vert[0] = 1.0;add_vert(vert);             /* place 1,0,0 in list */
    vert[1] = 1.0;add_vert(vert);             /* place 1,1,0 in list */
    vert[2] = 1.0;add_vert(vert);             /* place 1,1,1 in list */
    vert[1] = 0.0;add_vert(vert);             /* place 1,0,1 in list */
    vert[2] = 0.0;add_vert(vert);             /* place 1,0,0 in list */
    vert[1] = 1.0;add_vert(vert);             /* place 1,1,0 in list */
    vert[0] = 0.0;add_vert(vert);             /* place 0,1,0 in list */
    vert[2] = 1.0;add_vert(vert);             /* place 0,1,1 in list */
    vert[0] = 1.0;add_vert(vert);             /* place 1,1,1 in list */
    vert[1] = 0.0;add_vert(vert);             /* place 1,0,1 in list */
    vert[0] = 0.0;add_vert(vert);             /* place 0,0,1 in list */
#if defined _POV_ || defined VRML
    if (docell !=0){
#ifdef VRML
	fprintf(fpout,"Separator{\n Material{ \n diffuseColor 0.1 0.1 0.1\n }\n");
	fprintf(fpout,"Coordinate3{ point[\n");
	fprintf(fpout,"%f %f %f,\n",s_vert[0],s_vert[1],s_vert[2]);
	fprintf(fpout,"%f %f %f,\n",s_vert[3],s_vert[4],s_vert[5]);
	fprintf(fpout,"%f %f %f,\n",s_vert[6],s_vert[7],s_vert[8]);
	fprintf(fpout,"%f %f %f,\n",s_vert[9],s_vert[10],s_vert[11]);
	fprintf(fpout,"%f %f %f,\n",s_vert[15],s_vert[16],s_vert[17]);
	fprintf(fpout,"%f %f %f,\n",s_vert[18],s_vert[19],s_vert[20]);
	fprintf(fpout,"%f %f %f,\n",s_vert[21],s_vert[22],s_vert[23]);
	fprintf(fpout,"%f %f %f]\n}\n",s_vert[24],s_vert[25],s_vert[26]);
       fprintf(fpout,"IndexedLineSet{coordIndex[\n 0,1,2,3,0,4,5,6,7,4,-1,5,3,-1,2,6,-1,7,1,-1]\n}\n}\n");
      }
#else
    fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[0],s_vert[1],s_vert[2],s_vert[3],s_vert[4],s_vert[5],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
    fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[3],s_vert[4],s_vert[5],s_vert[6],s_vert[7],s_vert[8],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
    fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[6],s_vert[7],s_vert[8],s_vert[9],s_vert[10],s_vert[11],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
    fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[9],s_vert[10],s_vert[11],s_vert[12],s_vert[13],s_vert[14],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
    fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[12],s_vert[13],s_vert[14],s_vert[15],s_vert[16],s_vert[17],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
    fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[15],s_vert[16],s_vert[17],s_vert[18],s_vert[19],s_vert[20],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
    fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[18],s_vert[19],s_vert[20],s_vert[21],s_vert[22],s_vert[23],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
    fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[21],s_vert[22],s_vert[23],s_vert[24],s_vert[25],s_vert[26],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
        fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[24],s_vert[25],s_vert[26],s_vert[27],s_vert[28],s_vert[29],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
        fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[30],s_vert[31],s_vert[32],s_vert[33],s_vert[34],s_vert[35],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
        fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[36],s_vert[37],s_vert[38],s_vert[39],s_vert[40],s_vert[41],rad_cell);
    fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
        fprintf(fpout,"cylinder{< %f, %f, %f>, <%f, %f, %f>, %f \n",
        s_vert[24],s_vert[25],s_vert[26],s_vert[3],s_vert[4],s_vert[5],rad_cell);
     fprintf(fpout,"texture{pigment{color %s }}}\n",col_cell);
    }
#endif
    addflag=0;    
    nvert = 0;                  /* empty vertex list */
#else
/* create empty object */
    obj = GEOMcreate_obj(GEOM_POLYTRI,GEOM_NULL);
/* add polyline data and add it to edit list */
    if (docell != 0)    /* add the object if to be drawn */
        GEOMadd_polyline(obj,s_vert,GEOM_NULL,nvert,GEOM_COPY_DATA);
    GEOMedit_geometry(*output,"unit-cell",obj);
    GEOMedit_color(*output,"camera1",white);
    GEOMedit_color(*output,"unit-cell",black);
    GEOMedit_parent(*output,"unit-cell",File_Name);
    GEOMedit_transform_mode(*output,"unit-cell","parent",GEOM_NULL);  /* transform with parent */
    GEOMdestroy_obj(obj);       /* destroy object */
    nvert = 0;                  /* empty vertex list */
#endif
}

/* ************************************************************** */
/* ************************************************************** */
void find_all_in_box(i)

/* routine to locate all atoms of type 'i' in the display box */

int i;

{
    float vert[3];              /* storage for center */
    int ix[3];                  /* array for lattice offset */
    int j,k;

    expand_atom(i);         /* generate all atoms of this flavor in unit cell */
    for (ix[0] = -3;ix[0]<=3;++ix[0]) { /* step through cells parallel a */
	for (ix[1] = -3;ix[1]<=3;++ix[1]) { /* parallel b */
	    for (ix[2] = -3;ix[2]<=3;++ix[2]) { /* parallel c */
		for (j=0;j<ncell;++j) { /*atoms in cell*/
		    for (k=0;k<=2;++k) 
			vert[k] = ix[k] + cell_xyz[j][k];
		    add_vert(vert);  /* add point to cell if inside box*/
		}
	    }
	}
    }
}

/* ************************************************************** */
/* ************************************************************** */
#if defined _POV_ || defined VRML
void generate_spheres()
#else
void generate_spheres(output)
GEOMedit_list *output;
#endif

/* routine to generate spherical objects and add then to edit list */

{
    float *radii;                     /* dynamic array for sphere radii */  
    int i,j,k;                        /* loop counters */
#if !defined _POV_ && !defined VRML
    GEOMobj *obj;                     /* geometry object */
    char sphname[8];
    strcpy(sphname, "sphere1");
#endif
    for (i=1;i<nsphere;++i)     /* loop through spheres */
    {   nvert = 0;              /* clear the vertex list */
	for (j=0;j<natom;++j)   /* loop through atoms */
	   if( atom_n[j] == -i) /* negative means spherical atom */
	       find_all_in_box(j); /* generate all spheres for this atom */
	if (nvert > 0)      /* if spheres present */
	{  
/* allocate storage for radii of spheres */
	    radii = (float *) malloc ((unsigned)(nvert*sizeof(float)));
	    for (k=0;k<nvert;++k){ /* copy radii info to storage*/
		radii[k] = sphere_size[i] * Sphere_Mult;
#if defined _POV_ || defined VRML
#ifdef _POV_
                fprintf(fpout,"/* Sphere for %c%c at %f %f %f */ \n",sphere_l[i][0],sphere_l[i][1],o_vert[3*k],o_vert[3*k+1],o_vert[3*k+2]);
                fprintf(fpout,"sphere{<%f, %f, %f>, %f\n",s_vert[3*k],s_vert[3*k+1],s_vert[3*k+2],radii[k]);
                fprintf(fpout,"texture{pigment{color %s }}}\n",sphere_col[i]);
#else
               fprintf(fpout,"# Sphere for %c%c at %f %f %f \n",sphere_l[i][0],sphere_l[i][1],o_vert[3*k],o_vert[3*k+1],o_vert[3*k+2]);
fprintf(fpout,"   Separator {\n        Transform {\n");
fprintf(fpout,"            translation %f %f %f\n",s_vert[3*k],s_vert[3*k+1],s_vert[3*k+2]);
fprintf(fpout,"        }\n        Material {\n            diffuseColor        %s\n",sphere_col[i]);
fprintf(fpout,"        }\n        Sphere {\n            radius     %f\n",radii[k]);
fprintf(fpout,"        }\n    }\n");
#endif
            }
#else
            }
	    obj = GEOMcreate_sphere(GEOM_NULL,s_vert,radii,
		       GEOM_NULL,GEOM_NULL,nvert,GEOM_COPY_DATA);
	    GEOMset_pickable(obj,1); /* set individual objects pickable */
            sphname[6] = i + 47;   /* ASCII character set assumed here !!!!!!!!! */
	    GEOMedit_geometry(*output,sphname,obj);  /* add to edit list */
	    GEOMedit_parent(*output,sphname,File_Name);
            GEOMedit_transform_mode(*output,sphname,"parent",GEOM_NULL);  /* transform with parent */
	    GEOMdestroy_obj(obj);  /* get space back */
#endif
	    free(radii);
	}
    }
}

/* ************************************************************** */
/* ************************************************************** */
float dist(j,k)

/* calculates distance between vertices at s_vert[j] and s_vert[k] */

int j,k;

{
    float d,t;
    int i;

    d = 0.0;
    for (i=0;i<=2;++i) {
	t = s_vert[3*j+i] - s_vert[3*k+i];
	d += t*t;
    }
    d = sqrt(d);
    return(d);
}

/* ************************************************************** */
/* ************************************************************** */

#if defined _POV_ || defined VRML

void generate_bonds()

/* routine to generate bond descriptions and add then to edit list */

#else

void generate_bonds(output)
GEOMedit_list *output;

#endif
/* routine to generate bond descriptions and add then to edit list */

#define TRIANGLES 58
#define TRI_VERTS 3*TRIANGLES

{
    int i,j,k;           /* loop counters */
    int N_Bond;          /* Counter for number of bonds of a type */
    int l1,l2,l3;        /* loop counters for matrix operations */
    int nvert1;          /* vertex counter */
    int same_type;      /* indicator that two atom types are same */
    int loop_start;     /* variable to indicate start of second loop */
    char bnd1[2],bnd2[2];/* temporary storage of bond labels */
    float d;            /* bond distance */
    float df[3];        /* vector difference */
    float beta;         /* rotation angle */
    float gamma;        /* another rotation angle */
#if !defined _POV_ && !defined VRML
    float c[TRI_VERTS];       /* transformed triangle vertex coordinates */
    float cp[TRI_VERTS];      /* final triangle vertex coordinates */
    float normal[TRI_VERTS];  /* transformed normals */
    float T[3][3];      /* transformation matrix */
    GEOMobj *obj;       /* GEOMetry storage object */
    char bname[6];

    /* coordinates of tri. verts. of prototype bond */
    static float tr_cd[TRI_VERTS] = 
       {1, 0, 1, 0.809017, 0.618033, 1, 1, 0, -1, 0.809017, 0.618033, -1,
	0.809017, 0.618033, -1, 0.809017, -0.618033, 1, 0.809017, -0.618033, 1,
	1, 0, 1, 0.809017, -0.618033, -1, 1, 0, -1, 1, 0, -1, 0.309017, -1, 1,
	0.309017, -1, 1, 0.809017, -0.618033, 1, 0.309017, -1, -1,
	0.809017, -0.618033, -1, 0.809017, -0.618033, -1, -0.309017, -1, 1,
       -0.309017, -1, 1, 0.309017, -1, 1, -0.309017, -1, -1, 0.309017, -1, -1,
	0.309017, -1, -1, -0.809017, -0.618033, 1, -0.809017, -0.618033, 1,
       -0.309017, -1, 1, -0.809017, -0.618033, -1, -0.309017, -1, -1,
       -0.309017, -1, -1, -1, 0, 1, -1, 0, 1, -0.809017, -0.618033, 1,
       -1, 0, -1, -0.809017, -0.618033, -1, -0.809017, -0.618033, -1,
       -0.809017, 0.618033, 1, -0.809017, 0.618033, 1, -1, 0, 1,
       -0.809017, 0.618033, -1, -1, 0, -1, -1, 0, -1, -0.309017, 1, 1,
       -0.309017, 1, 1, -0.809017, 0.618033, 1, -0.309017, 1, -1,
       -0.809017, 0.618033, -1, -0.809017, 0.618033, -1, 0.309017, 1, 1,
	0.309017, 1, 1, -0.309017, 1, 1, 0.309017, 1, -1, -0.309017, 1, -1,
       -0.309017, 1, -1, 0.809017, 0.618033, 1, 0.809017, 0.618033, 1,
	0.309017, 1, 1, 0.809017, 0.618033, -1, 0.309017, 1, -1  };
    /* normals to triangles */
    static float tr_normal[TRI_VERTS] = 
      { 0.955422, 0.295242, 0, 0.955422, 0.295242, 0, 0.955422, 0.295242, 0,
	0.955422, 0.295242, 0, 0.955422, 0.295242, 0, 0.955422, -0.295242, 0,
	0.955422, -0.295242, 0, 0.955422, -0.295242, 0, 0.955422, -0.295242, 0,
	0.955422, -0.295242, 0, 0.955422, -0.295242, 0, 0.607063, -0.794654, 0,
	0.607063, -0.794654, 0, 0.607063, -0.794654, 0, 0.607063, -0.794654, 0,
	0.607063, -0.794654, 0, 0.607063, -0.794654, 0, 0, -1, 0,
	0, -1, 0, 0, -1, 0, 0, -1, 0,
	0, -1, 0, 0, -1, 0, -0.607063, -0.794654, 0,
       -0.607063, -0.794654, 0, -0.607063, -0.794654, 0,-0.607063, -0.794654, 0,
       -0.607063, -0.794654, 0, -0.607063, -0.794654, 0,-0.955422, -0.295242, 0,
       -0.955422, -0.295242, 0, -0.955422, -0.295242, 0,-0.955422, -0.295242, 0,
       -0.955422, -0.295242, 0, -0.955422, -0.295242, 0, -0.955422, 0.295242, 0,
       -0.955422, 0.295242, 0, -0.955422, 0.295242, 0, -0.955422, 0.295242, 0,
       -0.955422, 0.295242, 0, -0.955422, 0.295242, 0, -0.607063, 0.794654, 0,
       -0.607063, 0.794654, 0, -0.607063, 0.794654, 0, -0.607063, 0.794654, 0,
       -0.607063, 0.794654, 0, -0.607063, 0.794654, 0, 0, 1, 0,
	0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0,
	0.607063, 0.794654, 0, 0.607063, 0.794654, 0, 0.607063, 0.794654, 0,
	0.607063, 0.794654, 0, 0.607063, 0.794654, 0};

    strcpy(bname, "bond1");
#endif
    if (nbond == 1) return;         /* exit if no work to be done */

    for (i=1;i<nbond;++i)              /* loop through bond types*/
    {   nvert = 0;                     /* empty the vertex list */
	for (j=0;j<natom;++j)          /* loop through atom types */
	{   if (atom_n[j] < 0)         /* negative number implies sphere */
	    {    if ((bond_l1[i][0] == sphere_l[-atom_n[j]][0]) &&
		     (bond_l1[i][1] == sphere_l[-atom_n[j]][1]))
			 find_all_in_box(j);    /* get all atoms of type j */
	    }
	}
	bnd1[0] = bond_l1[i][0];
	bnd1[1] = bond_l1[i][1];
	nvert1 = nvert;
	if ((bond_l1[i][0] == bond_l2[i][0]) &&
	    (bond_l1[i][1] == bond_l2[i][1]))
	    {   bnd2[0] = bond_l1[i][0];
		bnd2[1] = bond_l1[i][1];
		same_type = 1;/* they are same */
	    }
	else
	    {   bnd2[0] = bond_l2[i][0];
		bnd2[1] = bond_l2[i][1];
		same_type = 0; /* atoms i and j are not the same type */
		for (j=0;j<natom;++j)    /* loop through atom types */
		{   if (atom_n[j] < 0)   /* negative number implies sphere */
		    {    if ((bond_l2[i][0] == sphere_l[-atom_n[j]][0]) &&
			 (bond_l2[i][1] == sphere_l[-atom_n[j]][1]))
			     find_all_in_box(j);   /* get all atoms of type j */
		    }
		}
	    }
	if (((nvert1 > 0) && (nvert > nvert1)) || (same_type == 1))
	{
#if !defined _POV_ && !defined VRML
            obj = GEOMcreate_obj(GEOM_POLYTRI,GEOM_NULL);
#endif
	    N_Bond = 0;
	    for (j=0;j<nvert1;++j)        /* loop through atoms of type 1 */
	    {   if(same_type == 1) loop_start = j;
		else loop_start = nvert1;
		for (k=loop_start;k<nvert;++k)     /* loop on type 2 atoms  */
		{
		    d = dist(j,k);          /* get atomic distance */
		    if ((d >= bond_min[i]) && (d <= bond_max[i]))
		    {   /* here if distance within range */
			df[0] = df[1] = Bond_Mult * bond_size[i];  /* scaling */
			df[2] = 0.5 * d;

#if defined _POV_ || defined VRML
#ifdef _POV_
                        fprintf(fpout,"/* Bond between %c%c at %f %f %f and %c%c at %f %f %f*/\n",
                           bnd1[0],bnd1[1],o_vert[3*j],o_vert[3*j+1],o_vert[3*j+2],
                           bnd2[0],bnd2[1],o_vert[3*k],o_vert[3*k+1],o_vert[3*k+2]);
                        fprintf(fpout,"cylinder{ <%f, %f, %f>, <%f, %f, %f>, %f \n",
                           s_vert[3*j],s_vert[3*j+1],s_vert[3*j+2],s_vert[3*k],
                           s_vert[3*k+1],s_vert[3*k+2],df[0]);
                        fprintf(fpout,"texture{pigment{color %s }}}\n",col_bond[i]); 
			N_Bond++;
                    }
                }
            }
#else
/* VRML Cylinders*/
                        fprintf(fpout,"# Bond between %c%c at %f %f %f and %c%c at %f %f %f\n",
                           bnd1[0],bnd1[1],o_vert[3*j],o_vert[3*j+1],o_vert[3*j+2],
                           bnd2[0],bnd2[1],o_vert[3*k],o_vert[3*k+1],o_vert[3*k+2]);
			for (l1=0;l1<3;++l1)
			    df[l1] = s_vert[3*k+l1] - s_vert[3*j+l1];
/* calculate rotation matrices needed to rotate bond about Z onto Y-Z plane and then about X so that
     bond is parallel to Z.  Finally, apply inverse transformation to cylinder, which started parallel
     to X.  Resulting cylinder will be parallel to vector between atoms. */ 
			beta = atan2(df[0],df[1]);   /* rotation angle about Z (in radians) */
			gamma = sqrt(df[0]*df[0] + df[1]*df[1]);
			gamma = atan2(gamma,df[2]);  /* Rotation angle about X */
                  fprintf(fpout,"   Separator{\n        Transform{\n ");
  fprintf(fpout,"      translation %f %f %f\n }\n",(s_vert[3*j]+s_vert[3*k])/2., 
  (s_vert[3*j+1]+s_vert[3*k+1])/2.,(s_vert[3*j+2]+s_vert[3*k+2])/2.);               
fprintf(fpout,"   Rotation {\n     rotation 0 0 1 %f\n }\n",-beta);  
fprintf(fpout,"   Rotation {\n     rotation 1 0 0 %f\n }\n",1.57-gamma);    
fprintf(fpout,"        Material {\n            diffuseColor        %s\n",col_bond[i]);
fprintf(fpout,"        }\n        Cylinder {\n            parts  SIDES\n            radius     %f\n            height     %f\n",Bond_Mult * bond_size[i],d);
fprintf(fpout,"        }\n    }\n");
                    }
                }
            }
#endif            
#else
			N_Bond++;
			for (l1=0;l1<TRIANGLES;++l1)
/* scale tr_cd into c */
			     for (l3=0;l3<3;++l3)
				 c[3*l1+l3] = tr_cd[3*l1+l3]*df[l3];
			for (l1=0;l1<3;++l1)
			    df[l1] = s_vert[3*k+l1] - s_vert[3*j+l1];
/* calculate rotation matrices needed to rotate bond about Z onto Y-Z plane and then about X so that
     bond is parallel to Z.  Finally, apply inverse transformation to cylinder, which started parallel
     to X.  Resulting cylinder will be parallel to vector between atoms. */ 
			beta = atan2(df[0],df[1]);   /* rotation angle about Z (in radians) */
			gamma = sqrt(df[0]*df[0] + df[1]*df[1]);
			gamma = atan2(gamma,df[2]);  /* Rotation angle about X */
			T[0][0] = cos(beta);
			T[0][1] = sin(beta)*cos(gamma);
			T[0][2] = sin(beta)*sin(gamma);
			T[1][0] = -sin(beta);
			T[1][1] = cos(beta)*cos(gamma);
			T[1][2] = cos(beta)*sin(gamma);
			T[2][0] = 0.0;
			T[2][1] = -sin(gamma);
			T[2][2] = cos(gamma);
			for (l1=0;l1<TRIANGLES;++l1) /* form product CP = T*C */
			    for (l2=0;l2<3;++l2)
			    {   cp[l2+3*l1] = 0.0;
				for (l3=0;l3<3;++l3)
				    cp[l2+3*l1] += c[3*l1+l3]*T[l2][l3];
			    }
			for (l1=0;l1<TRIANGLES;++l1)   /* transform normals */
			    for (l2=0;l2<3;++l2)
			    {   normal[l2+3*l1] = 0.0;
				for (l3=0;l3<3;++l3)
				 normal[l2+3*l1]+= tr_normal[3*l1+l3]*T[l2][l3];
			    }
			for (l1=0;l1<TRIANGLES;++l1)
			    for (l2=0;l2<3;++l2) /* move midpoint of cylinder to midpoint of bond */
				cp[3*l1+l2] += 0.5*(s_vert[3*k+l2] + s_vert[3*j+l2]);
			GEOMadd_polytriangle(obj,cp,normal,GEOM_NULL,TRIANGLES,GEOM_COPY_DATA);
		    }
		}
	    }
	    GEOMset_pickable(obj,1);
	    bname[4] = i + 47;   /* ASCII assumed here !!!!!!!! */ 
            GEOMedit_geometry(*output,bname,obj);
            GEOMedit_parent(*output,bname,File_Name);
            GEOMedit_transform_mode(*output,bname,"parent",GEOM_NULL);  /* transform with parent */
	    GEOMdestroy_obj(obj);   /* get rid of object */
#endif
	}
    }
}

/* ************************************************************** */
/* ************************************************************** */

void expand_atom(natom)

/* routine to expand an atom position and obtain all equipoints within
   the unit cell */

int natom;      /* number of atom in list to expand */

{
    float xp[3],xpp[3];         /* user in generating position */
    int i,j,k,l;                /* loop variables */

    ncell = 0;                  /* clear list */
    for (i=0;i<ng;++i) {        /* symmetry operations */
	for (j=0;j<=2;++j) {
	    xp[j] = ts[i][j];
	    for (k=0;k<=2;++k)
		xp[j] = xp[j] + ss[i][j][k] * atom_xyz[natom][k];
	}
	for (k=0;k<nlat;++k) {  /*lattice points */
	    for (l=0;l<=2;++l)
		xpp[l] = xp[l] + lat_pos[k][l];
	    add_to_list(xpp);
	    if (ns== 0) {
		for (l=0;l<=2;++l)
		    xpp[l] = -xpp[l];
		add_to_list(xpp);
	    }
	}
    }
}

/* ************************************************************** */
/* ************************************************************** */
void add_to_list(xp)

/* add atom position to unit-cell list if unique */

float xp[3];

{
    int i,ind;

    for (i=0;i<=2;++i) { /* get coordinates between 0 and 1 */
	if (xp[i] >= 1.0) xp[i] = xp[i] - 1.0;
	if (xp[i] < 0.0)  xp[i] = xp[i] + 1.0;
    }
    ind = 0;
    for (i=0;i<ncell;++i)
	if ((fabs(cell_xyz[i][0] - xp[0]) <= 0.00002) &&
	    (fabs(cell_xyz[i][1] - xp[1]) <= 0.00002) &&
	    (fabs(cell_xyz[i][2] - xp[2]) <= 0.00002)) ind = 1; /*set if same*/
    if (ind == 0) {  /* true for new position */
	for (i=0;i<=2;++i)  /*copy to list */
	    cell_xyz[ncell][i] = xp[i];
	ncell++;
    }
}

/* ************************************************************** */
/* ************************************************************** */
#if defined _POV_ || defined VRML
void build_poly_list(numb_list,polyno)

/* routine to take list of 'numb_list' indices in 's_vert' and find
   each plane on surface of convex polyhedron.  It then builds a list of
   pointers to these vertices for drawing program.  The algorithm is as 
   follows:
   
   1.  If there are 4 vertices, the polyhedron is a tetrahedron and all
       combinations of 3 vertices constitute a face.

   2.  For more than 4 vertices, some planes will be interior.  These are
       found by choosing three points (indicated by i,j,k) and forming two
       vectors defined by V1 = P(j) - P(i) and V2 = P(k) - P(i).  For each
       additional point (l), a third vector V3 = P(l) - P(i) is formed and
       the volume enclosed by these three vectors V = V3 . V1 x V2 is
       calculated.  If this volume is zero, point l lies in the plane.  If
       the plane is on the surface of the polyhedron, all non-zero volumes
       must have the same sign, which may be positive or negative.
*/

int numb_list;          /* number of candidate vertices */
int polyno;             /* number of polygon - for polygon color */
#else
void build_poly_list(numb_list)

int numb_list;          /* number of candidate vertices */

#endif

{
    int i,j,k,l,m;                      /* local loop counters */
    float v1[3],v2[3],v3[3];            /* vectors used in calculation */
    float p[3];                         /* coordinates of point i */    
    float vol,sign_z;
    int ind;                            /* indicator of surface plane */
    int no[30];                         /* place to keep vertex id's */
    int nplane;                         /* number in plane */
    int imk,jmk,kmk;
    int ii,jj,ll;
    double phi,cosphi;
    float xm,ym,zm,vx,vy,vz,bm,phi0,vxs,vys,vzs,bms;
    int ns[30];

    for (i=0;i<numb_list-2;++i) {
	no[0] = vertex_list[i];                      /* first point */
	for (m=0;m<=2;++m) p[m] = s_vert[3*no[0]+m];
	for (j=i+1;j<numb_list-1;++j) {
	    no[1] = vertex_list[j];                  /* second point */
	    for (k=0;k<=2;++k)v1[k] = s_vert[3*no[1]+k] - p[k];
	    for (k=j+1;k<numb_list;++k) {
		if (numb_list == 4) { /* tetrahedron */
		    poly_list[draw_list++] = 3;  /* number of vertices for triangle */
		    poly_list[draw_list++] = vertex_list[i]+1;
		    poly_list[draw_list++] = vertex_list[j]+1;
		    poly_list[draw_list++] = vertex_list[k]+1;
#ifdef _POV_ 
                    imk=vertex_list[i];
                    jmk=vertex_list[j];
                    kmk=vertex_list[k];

                    fprintf(fpout,"triangle{< %f, %f, %f>,\n",
                      s_vert[3*imk],s_vert[3*imk+1],
                      s_vert[3*imk+2]);
                    fprintf(fpout,"<%f, %f, %f>, <%f, %f, %f>texture{pigment{color %s}}}\n",
                      s_vert[3*jmk],s_vert[3*jmk+1],s_vert[3*jmk+2],
                      s_vert[3*kmk],s_vert[3*kmk+1],s_vert[3*kmk+2],
                      poly_col[polyno]);
                    if (edges == 1)  {
                      fprintf(fpout,"cylinder { <%f, %f, %f>, <%f, %f, %f> , %f \n",
                        s_vert[3*imk],s_vert[3*imk+1],s_vert[3*imk+2],
                        s_vert[3*jmk],s_vert[3*jmk+1],s_vert[3*jmk+2],rad_edge);
                      fprintf(fpout,"texture{pigment{color %s}}}\n",col_edge);
                      fprintf(fpout,"cylinder { <%f, %f, %f>, <%f, %f, %f> , %f \n",
                        s_vert[3*jmk],s_vert[3*jmk+1],s_vert[3*jmk+2],
                        s_vert[3*kmk],s_vert[3*kmk+1],s_vert[3*kmk+2],rad_edge);
                      fprintf(fpout,"texture{pigment{color %s}}}\n",col_edge);
                      fprintf(fpout,"cylinder { <%f, %f, %f>, <%f, %f, %f> , %f \n",
                        s_vert[3*kmk],s_vert[3*kmk+1],s_vert[3*kmk+2],
                        s_vert[3*imk],s_vert[3*imk+1],s_vert[3*imk+2],rad_edge);
                      fprintf(fpout,"texture{pigment{color %s}}}\n",col_edge);
                    }
#endif
		} else {
		    no[2] = vertex_list[k];          /* third point */
		    nplane = 3;
		    for (l=0;l<=2;++l)v2[l] = s_vert[3*no[2]+l]
		     - p[l];            /* get v2 */
		    sign_z = 0.0;
		    ind = 0;
		    v3[0] = v1[1] * v2[2] - v1[2] * v2[1];
		    v3[1] = v1[2] * v2[0] - v1[0] * v2[2];
		    v3[2] = v1[0] * v2[1] - v1[1] * v2[0];
		    for (l=0;l<numb_list;++l) {
			vol = 0.0;
			for (m=0;m<=2;++m) vol = vol + 
			    (s_vert[3*vertex_list[l]+m] - p[m]) * v3[m];
			if (fabs(vol) < 0.1) {
 			    if (l > k ) { /* MK */
				no[nplane++] = vertex_list[l]; /* add to plane list */
			        fprintf(flout,"Warning - new vertex added to plane list\n");
			    } else {
				if (( l != i) && (l != j) && (l != k)) {
				    ind = 1;  /* we already have this plane */
				    l = numb_list; /* skip rest of loop */
				}
			    }
			} else {
/* point is not in plane - check if first out-of-plane point found */
			    if (sign_z == 0.0) {
				sign_z = vol;   /* yes - set sign */
			    } else {
/* if here, not first out of plane - check that vol and sign_z have same sign. 
   If not, set ind=1 to indicate not a plane */
				if (vol*sign_z < 0.0) {
				    ind = 1;
				    l = numb_list; /* skip rest of inner loop */
				}
			    }
			}
		    } /* end of loop on l */
		    if (ind == 0) { /* 0 means a surface plane */
			poly_list[draw_list++] = nplane;  /* number of vertices in plane */
			for (l=0;l<nplane;++l)
			    poly_list[draw_list++] = no[l]+1;

#ifdef _POV_
                       fprintf(fpout,"/* %d-sided polygon */\n",nplane); 
#endif
/* Make vertices consecutive - find center of polygon */
#if defined _POV_ || defined VRML
                        xm=ym=zm=0.;
                        for (ii=0;ii<nplane;ii++){
                          xm=xm+s_vert[3*no[ii]];
                          ym=ym+s_vert[3*no[ii]+1];
                          zm=zm+s_vert[3*no[ii]+2];
                        }

                        xm=xm/ (float) nplane;
                        ym=ym/ (float) nplane;
                        zm=zm/ (float) nplane;

/* sort vertices in consecutive order based on minimal angles between vectors */

                        ns[0]=no[0];
                        for (ii=0;ii<nplane-1;ii++) {    
                          phi0=1000.;
                          vxs= s_vert[3*ns[ii]]-xm;
                          vys= s_vert[3*ns[ii]+1]-ym;
                          vzs= s_vert[3*ns[ii]+2]-zm;
                          bms= sqrt( vxs*vxs+vys*vys+vzs*vzs);

                          for (jj=0;jj<nplane;jj++) { 

                            for(ll=0;ll<ii;ll++) 
                              if (ns[ll]== no[jj]) jj++;

                              if (ns[ii]!=no[jj] && jj <nplane){

                                vx= s_vert[3*no[jj]]-xm;
                                vy= s_vert[3*no[jj]+1]-ym;
                                vz= s_vert[3*no[jj]+2]-zm;
                                bm= sqrt( vx*vx+vy*vy+vz*vz);

                                cosphi=( vxs* vx + vys*vy+vzs*vz) /(bms*bm);
                                if (cosphi < -1. ) cosphi=-1.;
                                if (cosphi > 1. ) cosphi=1.;

                                phi = acos(cosphi)*180/3.1415926;

                                if (phi <phi0){
                                  phi0=phi;
                                  ns[ii+1]=no[jj];
                                }
                              }
                            } 
                          } 
#ifdef VRML
if (numb_list!=4){
                       draw_list=draw_list-nplane;
			for (l=0;l<nplane;++l)
			    poly_list[draw_list++] = ns[l]+1;
}
#else
                          for (l=0;l<nplane-2;++l){
                            fprintf(fpout,"triangle {< %f, %f, %f>,\n"
                              ,s_vert[3*ns[0]],s_vert[3*ns[0]+1],s_vert[3*ns[0]+2]);
                            fprintf(fpout,"<%f, %f, %f>, <%f, %f, %f>texture{pigment{color %s}}}\n",
                               s_vert[3*ns[l+1]],s_vert[3*ns[l+1]+1],s_vert[3*ns[l+1]+2],
                               s_vert[3*ns[l+2]],s_vert[3*ns[l+2]+1],s_vert[3*ns[l+2]+2],
                               poly_col[polyno]);
                          }
                          if (edges == 1)  {
                            for (l=0;l<nplane-1;++l){
                              fprintf(fpout,"cylinder { <%f, %f, %f>, <%f, %f, %f> , %f \n",
                                s_vert[3*ns[l]],s_vert[3*ns[l]+1],s_vert[3*ns[l]+2],
                                s_vert[3*ns[l+1]],s_vert[3*ns[l+1]+1],s_vert[3*ns[l+1]+2],rad_edge);
                              fprintf(fpout,"texture{pigment{color %s}}}\n",col_edge);
                          } 
                          fprintf(fpout,"cylinder { <%f, %f, %f>, <%f, %f, %f> , %f \n",
                            s_vert[3*ns[0]],s_vert[3*ns[0]+1],s_vert[3*ns[0]+2],
                            s_vert[3*ns[nplane-1]],s_vert[3*ns[nplane-1]+1],s_vert[3*ns[nplane-1]+2],rad_edge);
                          fprintf(fpout,"texture{pigment{color %s}}}\n",col_edge);
                        }
#endif
#endif 
                    }
		} /* end of numb_list != 4 */
	    }   /* end of for (k.... */
	}
    } 

return;
} /* end of poly_build_list */

/* ************************************************************** */
/* ************************************************************** */
#if defined _POV_ || defined VRML
void generate_poly()
#else
void generate_poly(output)
GEOMedit_list *output;
#endif

/* procedure to generate vertices of polyhedra and add to edit list.  Each
   atom type creates a separate object */

{
    int i,j,k;                  /* loop counters, etc */
    int nvert_start;            /* starting number for cations */
    int polyno;                 /* number of current polyhedron */
#if  !defined _POV_ && !defined VRML
    int nvert;                  /* total number of vertices */
    GEOMobj *obj;               /* GEOMetry storage object */
    char pname[6];

    strcpy(pname,"poly1");
#endif
    if (npoly == 1) return;     /* exit NOW if no work */
    for (i=0;i<3;++i) boxlim[i] += 3.0; /* expand box for cation generation */
    nvert = 0;                 /* make list empty */
    for (i=0;i<natom;++i)       /* loop through atoms */
	if (atom_n[i] == 0) find_all_in_box(i); /* generate all cations */
    for (i=0;i<3;++i) boxlim[i] -= 3.0; /*shrink box to original size */
    if (nvert == 0) return;     /* if no anions, no polyhedra */
    nvert_start = nvert;        /* save start of cation list */
    for (i=0;i<natom;++i) {     /* start loop for cations */
	nvert = nvert_start;    /* get rid of previous cations */
	if (atom_n[i] > 0 && atom_n[i] <10000) {    /* greater than zero for polyhedral centers */
#if !defined _POV_ && !defined VRML
	    obj = GEOMcreate_obj(GEOM_POLYHEDRON,GEOM_NULL); /* create display object */
	    GEOMadd_vertices(obj,s_vert,nvert_start,GEOM_COPY_DATA); /* add anion vertices */
#endif
	    polyno = atom_n[i];  /* save polyhedron number */
	    find_all_in_box(i);  /* add all polyhedral centers in box */
	    if(nvert_start < nvert) { /* check for some found */
		draw_list = 0;  /* initialize the draw list */
		for (j=nvert_start;j<nvert;++j) { /* loop through cations */
		    numb_list = 0;      /* list clear */
		    for (k=0;k<nvert_start;++k) { /* loop through anions */
			if (dist(j,k) <= poly_size[polyno])  /* add vertex k to poly list */
			    vertex_list[numb_list++] = k;
		    }
		    if (numb_list > 3) {         /* only do if at least a tetrahedron */
#if defined _POV_ || defined VRML
                        fprintf(flout,"Generated polyhedron with %d vertices around %c%c at %f %f %f \n"
                          ,numb_list,atom_l[i][0],atom_l[i][1],s_vert[3*j],s_vert[3*j+1],s_vert[3*j+2]);
#ifdef VRML
       	                fprintf(fpout,"# Polyhedron (%d) around %c%c at %f %f %f \n"
                          ,numb_list,atom_l[i][0],atom_l[i][1],o_vert[3*j],o_vert[3*j+1],o_vert[3*j+2]);
fprintf(fpout,"    Separator {\n	");
fprintf(fpout,"        Material {\n            diffuseColor        %s \n}\n",poly_col[polyno]);
			build_poly_list(numb_list,polyno); /* select polygons */
			poly_list[draw_list++] = 0;       /* terminate polygon list */
                        plot_vrml_poly();
#else
       	                fprintf(fpout,"/* Polyhedron (%d) around %c%c at %f %f %f */ \n"
                          ,numb_list,atom_l[i][0],atom_l[i][1],o_vert[3*j],o_vert[3*j+1],o_vert[3*j+2]);
			build_poly_list(numb_list,polyno); /* select polygons */
			poly_list[draw_list++] = 0;       /* terminate polygon list */
#endif
#else
			build_poly_list(numb_list); /* select polygons */
			poly_list[draw_list++] = 0;       /* terminate polygon list */
			GEOMadd_polygons(obj,poly_list,GEOM_CONVEX,GEOM_COPY_DATA);
#endif
			draw_list = 0;
		    } 
		}
	   }
#if !defined _POV_ && !defined VRML
#ifndef OSF1
	   GEOMgen_normals(obj,GEOM_FACET_NORMALS);
#endif
	   GEOMcvt_polyh_to_polytri(obj,GEOM_SURFACE);
	   GEOMset_pickable(obj,1); /* set individual objects pickable */
           pname[4] = i + 48;    /* ASCII assumed !!!!! */
           GEOMedit_geometry(*output,pname,obj);
           GEOMedit_parent(*output,pname,File_Name);
           GEOMedit_transform_mode(*output,pname,"parent",GEOM_NULL);  /* transform with parent */
	   GEOMdestroy_obj(obj);   /* get rid of this object */
#endif
       }
    } /* end of loop through cations */
}

void generate_planes()

/* procedure to generate vertices of polyhedra and add to edit list.  Each
   atom type creates a separate object */

{
    int i,j,k;                  /* loop counters, etc */
    int nvert_start;            /* starting number for cations */
    int planeno;                 /* number of current polyhedron */
    int ii,jj,ll,l;
    int no[30],ns[30];
    float xm,ym,zm,vx,vy,vz,bm,vxs,vys,vzs,bms,phi,phi0,cosphi;

    if (nplane == 1) return;     /* exit NOW if no work */
    for (i=0;i<3;++i) boxlim[i] += 3.0; /* expand box for cation generation */
    nvert = 0;                 /* make list empty */
    for (i=0;i<natom;++i)       /* loop through atoms */
	if (atom_n[i] == 0) find_all_in_box(i); /* generate all cations */
    for (i=0;i<3;++i) boxlim[i] -= 3.0; /*shrink box to original size */
    if (nvert == 0) return;     /* if no anions, no polyhedra */
    nvert_start = nvert;        /* save start of cation list */
    for (i=0;i<natom;++i) {     /* start loop for cations */
	nvert = nvert_start;    /* get rid of previous cations */
	if (atom_n[i] > 10000) {    /* greater than zero for polyhedral centers */

	    planeno = atom_n[i]-10000;  /* save polyhedron number */
	    find_all_in_box(i);  /* add all polyhedral centers in box */
	    if(nvert_start < nvert) { /* check for some found */
		draw_list = 0;  /* initialize the draw list */
		for (j=nvert_start;j<nvert;++j) { /* loop through cations */
		    numb_list = 0;      /* list clear */
		    for (k=0;k<nvert_start;++k) { /* loop through anions */
			if (dist(j,k) <= plane_size[planeno])  /* add vertex k to poly list */
			    vertex_list[numb_list++] = k;
		    }
		    if (numb_list > 2) {         /* only do if at least a triangle*/
#if defined _POV_ || defined VRML
                        fprintf(flout,"Generated polygon with %d vertices around %c%c at %f %f %f \n"
                          ,numb_list,atom_l[i][0],atom_l[i][1],s_vert[3*j],s_vert[3*j+1],s_vert[3*j+2]);
#ifdef VRML
       	                fprintf(fpout,"# Polygon (%d) around %c%c at %f %f %f \n"
                          ,numb_list,atom_l[i][0],atom_l[i][1],o_vert[3*j],o_vert[3*j+1],o_vert[3*j+2]);
fprintf(fpout,"    Separator {\n	");
fprintf(fpout,"        Material {\n            diffuseColor        %s \n}\n",plane_col[planeno]);
#endif
                        poly_list[0]=numb_list;
/* Make vertices consecutive - find center of polygon */
#if defined _POV_ || defined VRML
                        xm=ym=zm=0.;
                        for (ii=0;ii<numb_list;ii++){
                         no[ii]=vertex_list[ii];
                          xm=xm+s_vert[3*no[ii]];
                          ym=ym+s_vert[3*no[ii]+1];
                          zm=zm+s_vert[3*no[ii]+2];
                        }

                        xm=xm/ (float) numb_list;
                        ym=ym/ (float) numb_list;
                        zm=zm/ (float) numb_list;

/* sort vertices in consecutive order based on minimal angles between vectors */

                        ns[0]=no[0];
                        for (ii=0;ii<numb_list-1;ii++) {    
                          phi0=1000.;
                          vxs= s_vert[3*ns[ii]]-xm;
                          vys= s_vert[3*ns[ii]+1]-ym;
                          vzs= s_vert[3*ns[ii]+2]-zm;
                          bms= sqrt( vxs*vxs+vys*vys+vzs*vzs);

                          for (jj=0;jj<numb_list;jj++) { 

                            for(ll=0;ll<ii;ll++) 
                              if (ns[ll]== no[jj]) jj++;

                              if (ns[ii]!=no[jj] && jj <numb_list){

                                vx= s_vert[3*no[jj]]-xm;
                                vy= s_vert[3*no[jj]+1]-ym;
                                vz= s_vert[3*no[jj]+2]-zm;
                                bm= sqrt( vx*vx+vy*vy+vz*vz);

                                cosphi=( vxs* vx + vys*vy+vzs*vz) /(bms*bm);
                                if (cosphi < -1. ) cosphi=-1.;
                                if (cosphi > 1. ) cosphi=1.;

                                phi = acos(cosphi)*180/3.1415926;

                                if (phi <phi0){
                                  phi0=phi;
                                  ns[ii+1]=no[jj];
                                }
                              }
                            } 
                          } 

#ifdef VRML
                         draw_list=1;
                        poly_list[0]=numb_list;
			for (l=0;l<numb_list;++l)
			    poly_list[draw_list++] = ns[l]+1;
			poly_list[draw_list++] = 0;       /* terminate polygon list */
                        plot_vrml_poly();
#else
       	                fprintf(fpout,"/* Polygon (%d) around %c%c at %f %f %f */ \n"
                          ,numb_list,atom_l[i][0],atom_l[i][1],o_vert[3*j],o_vert[3*j+1],o_vert[3*j+2]);
                          for (l=0;l<numb_list-2;++l){
                            fprintf(fpout,"triangle {< %f, %f, %f>,\n"
                              ,s_vert[3*ns[0]],s_vert[3*ns[0]+1],s_vert[3*ns[0]+2]);
                            fprintf(fpout,"<%f, %f, %f>, <%f, %f, %f>texture{pigment{color %s}}}\n",
                               s_vert[3*ns[l+1]],s_vert[3*ns[l+1]+1],s_vert[3*ns[l+1]+2],
                               s_vert[3*ns[l+2]],s_vert[3*ns[l+2]+1],s_vert[3*ns[l+2]+2],
                               plane_col[planeno]);
                          }
                          if (edges == 1)  {
                            for (l=0;l<numb_list-1;++l){
                              fprintf(fpout,"cylinder { <%f, %f, %f>, <%f, %f, %f> , %f \n",
                                s_vert[3*ns[l]],s_vert[3*ns[l]+1],s_vert[3*ns[l]+2],
                                s_vert[3*ns[l+1]],s_vert[3*ns[l+1]+1],s_vert[3*ns[l+1]+2],rad_edge);
                              fprintf(fpout,"texture{pigment{color %s}}}\n",col_edge);
                          } 
                          fprintf(fpout,"cylinder { <%f, %f, %f>, <%f, %f, %f> , %f \n",
                            s_vert[3*ns[0]],s_vert[3*ns[0]+1],s_vert[3*ns[0]+2],
                            s_vert[3*ns[numb_list-1]],s_vert[3*ns[numb_list-1]+1],s_vert[3*ns[numb_list-1]+2],rad_edge);
                          fprintf(fpout,"texture{pigment{color %s}}}\n",col_edge);
                        }
#endif
#endif
			draw_list = 0;
		    } 
		}
	   }
       }
    } /* end of loop through cations */
}
#endif

#ifdef VRML
void plot_vrml_poly()
{
int i,j,k,l ;

	fprintf(fpout,"Coordinate3 { point [\n");
        for (i=0;i<draw_list;i++){
         j=poly_list[i];
/*        fprintf(fpout,"# %d-sided polygon:\n",j); */
         for (k=i+1;k<=i+j;k++){
         l=poly_list[k]-1;
         if(k<draw_list-2){
          fprintf(fpout," %f %f %f,\n",s_vert[3*l],s_vert[3*l+1],s_vert[3*l+2]);
         }else{
         fprintf(fpout," %f %f %f]\n",s_vert[3*l],s_vert[3*l+1],s_vert[3*l+2]);
         }
         }
        i=i+j;
        } 
	fprintf(fpout,"}\n IndexedFaceSet { coordIndex [\n");
        l=0;
        for (i=0;i<draw_list-1;i++){
         j=poly_list[i];
/*        fprintf(fpout,"# %d-sided polygon:\n",j); */
         for (k=0;k<j;k++){
	   fprintf(fpout,"%d,",l);
        	l++;
         }
         if (i+j<draw_list-2){        
         	   fprintf(fpout,"-1,\n");
			}else{
 		   fprintf(fpout,"-1]\n  }\n");
		}
		i=i+j;
	}
	if (edges==1){
	fprintf(fpout," Material {\n   diffuseColor %s \n }\n",col_edge);
	fprintf(fpout,"IndexedLineSet { coordIndex [\n");
        l=0;
        for (i=0;i<draw_list-1;i++){
         j=poly_list[i];
         for (k=0;k<j;k++){
	   fprintf(fpout,"%d,",l);
        	l++;
         }
         if (i+j<draw_list-2){        
         	   fprintf(fpout,"%d,-1,\n",l-k);
			}else{
 		   fprintf(fpout,"%d,-1]\n  }\n }\n",l-k);
		}
		i=i+j;
  	}
       }else{
       fprintf(fpout," }\n");
        }
	return;
}
#endif
		

